@@ -5,39 +5,53 @@ import itertools
 
 from doit import dependency
 from doit.task import Task
-from doit.main import TaskSetup, InvalidCommand
-from doit.runner import run_tasks
+from doit.main import TaskControl, InvalidCommand
+from doit.runner import Runner, MP_Runner
 from doit.reporter import REPORTERS
 from doit.dependency import Dependency
 
 
 def doit_run(dependencyFile, task_list, output, options=None,
              verbosity=None, alwaysExecute=False, continue_=False,
-             reporter='default'):
+             reporter='default', num_process=0):
     # get tasks to be executed
-    selected_tasks = TaskSetup(task_list, options).process()
+    task_control = TaskControl(task_list)
+    task_control.process(options)
+
+    # reporter
     if reporter not in REPORTERS:
         msg = ("No reporter named '%s'.\nType 'doit help run' to see a list "
                "of available reporters.")
         raise InvalidCommand(msg % reporter)
     reporter_cls = REPORTERS[reporter]
 
+    # verbosity
     if verbosity is None:
         use_verbosity = Task.DEFAULT_VERBOSITY
     else:
         use_verbosity = verbosity
     show_out = use_verbosity < 2 # show on error report
 
+    # outstream
     if isinstance(output, str):
         outstream = open(output, 'w')
     else: # outfile is a file-like object (like StringIO or sys.stdout)
         outstream = output
+
+    # run
     try:
         # FIXME stderr will be shown twice in case of task error/failure
         reporter_obj = reporter_cls(outstream, show_out , True)
 
-        return run_tasks(dependencyFile, selected_tasks, reporter_obj,
-                         verbosity, alwaysExecute, continue_)
+        if num_process == 0:
+            runner = Runner(dependencyFile, reporter_obj, continue_,
+                            alwaysExecute, verbosity)
+        else:
+            runner = MP_Runner(dependencyFile, reporter_obj, continue_,
+                               alwaysExecute, verbosity, num_process)
+
+        runner.run_tasks(task_control)
+        return runner.finish()
     finally:
         if isinstance(output, str):
             outstream.close()
@@ -77,7 +91,7 @@ def doit_clean(task_list, outstream, dryrun, clean_dep, clean_tasks):
 
 def doit_list(dependencyFile, task_list, outstream, filter_tasks,
               print_subtasks=False, print_doc=False, print_status=False,
-              print_private=False):
+              print_private=False, print_dependencies=False):
     """List task generators, in the order they were defined.
 
     @param filter_tasks (list -str): print only tasks from this list
@@ -86,6 +100,7 @@ def doit_list(dependencyFile, task_list, outstream, filter_tasks,
     @param print_doc(bool)
     @param print_status(bool)
     @param print_private(bool)
+    @param print_dependencies(bool)
     """
     status_map = {'ignore': 'I', 'up-to-date': 'U', 'run': 'R'}
     def _list_print_task(task):
@@ -99,6 +114,13 @@ def doit_list(dependencyFile, task_list, outstream, filter_tasks,
             task_str = "%s %s" % (status_map[task_uptodate], task_str)
 
         outstream.write("%s\n" % task_str)
+
+        # print dependencies
+        if print_dependencies:
+            for dep in task.file_dep:
+                outstream.write(" -  %s\n" % dep)
+            outstream.write("\n")
+
         # print subtasks
         if print_subtasks:
             for subt in task.task_dep:
@@ -242,13 +264,20 @@ def doit_auto(dependency_file, task_list, filter_tasks, loop_callback=None):
     @param filter_tasks (list -str): print only tasks from this list
     @loop_callback: used to stop loop on unittests
     """
-    selected_tasks = TaskSetup(task_list, filter_tasks).process()
-    watch_files = list(itertools.chain(*[s.file_dep for s in selected_tasks]))
+    task_control = TaskControl(task_list)
+    task_control.process(filter_tasks)
+    tasks_to_run = list(set([t for t in task_control.get_next_task(True)]))
+    watch_tasks = [t.name for t in tasks_to_run]
+    watch_files = list(itertools.chain(*[s.file_dep for s in tasks_to_run]))
+    watch_files = list(set(watch_files))
 
     class DoitAutoRun(FileModifyWatcher):
         def handle_event(self, event):
             doit_run(dependency_file, task_list, sys.stdout,
-                     filter_tasks, reporter='executed-only')
+                     watch_tasks, reporter='executed-only')
+            # reset run_status
+            for task in task_list:
+                task.run_status = None
 
     fw = DoitAutoRun(watch_files)
     # always run once when started
