@@ -88,6 +88,17 @@ opt_continue = {'name': 'continue',
                         "[default: %(default)s]"
                 }
 
+
+opt_num_process = {'name': 'num_process',
+                   'short': 'n',
+                   'long': 'process',
+                   'type': int,
+                   'default': 0,
+                   'help': "number of subprocesses"
+                   "[default: %(default)s]"
+                   }
+
+
 # verbosity
 opt_verbosity = {'name':'verbosity',
                  'short':'v',
@@ -166,7 +177,8 @@ def cmd_run(params, args):
         return doit_run(params['dep_file'], dodo_tasks['task_list'],
                         params['outfile'], default_tasks,
                         params['verbosity'], params['always'],
-                        params['continue'], params['reporter'])
+                        params['continue'], params['reporter'],
+                        params['num_process'])
 
     # explicit sub-cmd. parse arguments again
     commands = params['sub']
@@ -211,6 +223,13 @@ opt_list_private = {'name': 'private',
                     'default': False,
                     'help': "print private tasks (start with '_')"}
 
+opt_list_dependencies = {'name': 'list_deps',
+                         'short': '',
+                         'long': 'deps',
+                         'type': bool,
+                         'default': False,
+                         'help': "print list of dependencies (file dependencies only)"}
+
 
 def cmd_list(params, args):
     dodo_tasks = main.get_tasks(params['dodoFile'], params['cwdPath'],
@@ -218,7 +237,7 @@ def cmd_list(params, args):
     params.update_defaults(dodo_tasks['config'])
     return doit_list(params['dep_file'], dodo_tasks['task_list'], sys.stdout,
                      args, params['all'], not params['quiet'],
-                     params['status'], params['private'])
+                     params['status'], params['private'], params['list_deps'])
 
 ##########################################################
 ########## clean
@@ -324,6 +343,7 @@ dependencies:
   - type: list. items:
     * file (string) path relative to the dodo file
     * task (string) ":<task_name>"
+    * TODO "?<task_name>"
     * run-once (True bool)
     * never up-to-date (False bool)
     * None values will be just ignored
@@ -390,7 +410,7 @@ def cmd_main(cmd_args):
     # run command
     run_options = (opt_version, opt_help, opt_dodo, opt_cwd, opt_depfile,
                    opt_always, opt_continue, opt_verbosity, opt_reporter,
-                   opt_outfile)
+                   opt_outfile, opt_num_process)
     subCmd['run'] = cmdparse.Command('run', run_options, cmd_run, run_doc)
 
     # clean command
@@ -400,7 +420,8 @@ def cmd_main(cmd_args):
 
     # list command
     list_options = (opt_dodo, opt_depfile, opt_cwd, opt_listall,
-                    opt_list_quiet, opt_list_status, opt_list_private)
+                    opt_list_quiet, opt_list_status, opt_list_private,
+                    opt_list_dependencies)
     subCmd['list'] = cmdparse.Command('list', list_options, cmd_list, list_doc)
 
     # forget command
