@@ -62,7 +62,7 @@ class TestInterface(object):
         mock_run = Mock()
         monkeypatch.setattr(doit_cmd, "doit_run", mock_run)
         doit_cmd.cmd_main(["--output-file", "mylog.txt",
-                                 "-v2", "--continue", "--reporter", "myr"])
+                         "-v2", "--continue", "--reporter", "myr", "-n", "4"])
 
         expected = [('dependencyFile', '.doit.db'),
                     ('task_list', mock_get_tasks()['task_list']),
@@ -71,7 +71,8 @@ class TestInterface(object):
                     ('verbosity', 2),
                     ('alwaysExecute', False),
                     ('continue_', True),
-                    ('reporter', "myr"),]
+                    ('reporter', "myr"),
+                    ('num_process', 4),]
 
         assert len(expected) == len(argspec[0])
         assert len(expected) == len(mock_run.call_args[0])
@@ -105,7 +106,8 @@ class TestInterface(object):
         monkeypatch.setattr(main, "get_tasks", mock_get_tasks)
         mock_list = Mock()
         monkeypatch.setattr(doit_cmd, "doit_list", mock_list)
-        doit_cmd.cmd_main(["list", "--all", "--quiet", "--status", "c", "a"])
+        doit_cmd.cmd_main(["list", "--all", "--quiet", "--status", "--deps",
+                           "c", "a"])
         assert mock_list.called
 
         expected = [('dependencyFile', '.doit.db'),
@@ -115,7 +117,8 @@ class TestInterface(object):
                     ('print_subtasks', True),
                     ('print_doc', False),
                     ('print_status', True),
-                    ('print_private', False),]
+                    ('print_private', False),
+                    ('print_dependencies', True),]
 
         assert len(expected) == len(argspec[0])
         assert len(expected) == len(mock_list.call_args[0])
