@@ -3,7 +3,7 @@ import StringIO
 
 from doit import reporter
 from doit.task import Task
-from doit import CatchedException
+from doit.exceptions import CatchedException
 from doit.dependency import json #FIXME move json import to __init__.py
 
 
@@ -44,8 +44,20 @@ class TestConsoleReporter(object):
         rep = reporter.ConsoleReporter(StringIO.StringIO(), True, True)
         exception = CatchedException("I got you")
         rep.cleanup_error(exception)
-        assert "I got you" in capsys.readouterr()[1]
+        err = capsys.readouterr()[1]
+        assert "I got you" in err
 
+    def test_teardownTask(self):
+        rep = reporter.ConsoleReporter(StringIO.StringIO(), True, True)
+        rep.teardown_task(Task("t_name", None))
+        # no output on teardown task
+        assert "" in rep.outstream.getvalue()
+
+    def test_addSuccess(self):
+        rep = reporter.ConsoleReporter(StringIO.StringIO(), True, True)
+        rep.add_success(Task("t_name", None))
+        # no output on success task
+        assert "" in rep.outstream.getvalue()
 
     def test_addFailure(self):
         rep = reporter.ConsoleReporter(StringIO.StringIO(), True, True)
@@ -121,13 +133,15 @@ class TestJsonReporter(object):
         rep.start_task(t4)
         rep.skip_ignore(t4)
 
+        # just ignore these
+        rep.cleanup_error(Exception('xx'))
+        rep.teardown_task(t4)
+
         rep.complete_run()
         got = json.loads(output.getvalue())
         for task_result in got['tasks']:
             assert expected[task_result['name']] == task_result['result'], got
 
-        # just ignore this
-        rep.cleanup_error(Exception('xx'))
 
 
     def test_ignore_stdout(self):
