@@ -1,10 +1,12 @@
 import os, shutil
-import sys, tempfile
+import tempfile
 from StringIO import StringIO
 
 import py.test
 
-from doit import TaskError, TaskFailed
+from doit.exceptions import TaskError
+from doit.exceptions import CatchedException
+from doit import action
 from doit import task
 
 #path to test folder
@@ -13,283 +15,6 @@ PROGRAM = "python %s/sample_process.py" % TEST_PATH
 
 
 
-def pytest_funcarg__tmpfile(request):
-    """crate a temporary file"""
-    return request.cached_setup(
-        setup=os.tmpfile,
-        teardown=(lambda tmpfile: tmpfile.close()),
-        scope="function")
-
-
-############# CmdAction
-class TestCmdAction(object):
-    # if nothing is raised it is successful
-    def test_success(self):
-        action = task.CmdAction(PROGRAM)
-        action.execute()
-
-    def test_error(self):
-        action = task.CmdAction("%s 1 2 3" % PROGRAM)
-        py.test.raises(TaskError, action.execute)
-
-    def test_failure(self):
-        action = task.CmdAction("%s please fail" % PROGRAM)
-        py.test.raises(TaskFailed, action.execute)
-
-    def test_str(self):
-        action = task.CmdAction(PROGRAM)
-        assert "Cmd: %s" % PROGRAM == str(action)
-
-    def test_repr(self):
-        action = task.CmdAction(PROGRAM)
-        expected = "<CmdAction: '%s'>" % PROGRAM
-        assert  expected == repr(action), repr(action)
-
-    def test_result(self):
-        action = task.CmdAction("%s 1 2" % PROGRAM)
-        action.execute()
-        assert "12" == action.result
-
-    def test_values(self):
-        # for cmdActions they are always empty
-        action = task.CmdAction("%s 1 2" % PROGRAM)
-        action.execute()
-        assert {} == action.values
-
-
-class TestCmdVerbosity(object):
-    # Capture stderr
-    def test_captureStderr(self):
-        cmd = "%s please fail" % PROGRAM
-        action = task.CmdAction(cmd)
-        py.test.raises(TaskFailed, action.execute)
-        assert "err output on failure" == action.err, repr(action.err)
-
-    # Capture stdout
-    def test_captureStdout(self):
-        action = task.CmdAction("%s hi_stdout hi2" % PROGRAM)
-        action.execute()
-        assert "hi_stdout" == action.out, repr(action.out)
-
-    # Do not capture stderr
-    # test using a tempfile. it is not possible (at least i dont know)
-    # how to test if the output went to the parent process,
-    # faking sys.stderr with a StringIO doesnt work.
-    def test_noCaptureStderr(self, tmpfile):
-        action = task.CmdAction("%s please fail" % PROGRAM)
-        py.test.raises(TaskFailed, action.execute, err=tmpfile)
-        tmpfile.seek(0)
-        got = tmpfile.read()
-        assert "err output on failure" == got, repr(got)
-        assert "err output on failure" == action.err, repr(action.err)
-
-    # Do not capture stdout
-    def test_noCaptureStdout(self, tmpfile):
-        action = task.CmdAction("%s hi_stdout hi2" % PROGRAM)
-        action.execute(out=tmpfile)
-        tmpfile.seek(0)
-        got = tmpfile.read()
-        assert "hi_stdout" == got, repr(got)
-        assert "hi_stdout" == action.out, repr(action.out)
-
-
-
-############# PythonAction
-
-class TestPythonAction(object):
-
-    def test_success_bool(self):
-        def success_sample():return True
-        action = task.PythonAction(success_sample)
-        # nothing raised it was successful
-        action.execute()
-
-    def test_success_None(self):
-        def success_sample():return
-        action = task.PythonAction(success_sample)
-        # nothing raised it was successful
-        action.execute()
-
-    def test_success_str(self):
-        def success_sample():return ""
-        action = task.PythonAction(success_sample)
-        # nothing raised it was successful
-        action.execute()
-
-    def test_success_dict(self):
-        def success_sample():return {}
-        action = task.PythonAction(success_sample)
-        # nothing raised it was successful
-        action.execute()
-
-    def test_error_object(self):
-        # anthing but None, bool, string or dict
-        def error_sample(): return object()
-        action = task.PythonAction(error_sample)
-        py.test.raises(TaskError, action.execute)
-
-    def test_error_exception(self):
-        def error_sample(): raise Exception("asdf")
-        action = task.PythonAction(error_sample)
-        py.test.raises(TaskError, action.execute)
-
-    def test_fail_bool(self):
-        def fail_sample():return False
-        action = task.PythonAction(fail_sample)
-        py.test.raises(TaskFailed, action.execute)
-
-    # any callable should work, not only functions
-    def test_nonFunction(self):
-        class CallMe:
-            def __call__(self):
-                return False
-
-        action= task.PythonAction(CallMe())
-        py.test.raises(TaskFailed, action.execute)
-
-    # helper to test callable with parameters
-    def _func_par(self,par1,par2,par3=5):
-        if par1 == par2 and par3 > 10:
-            return True
-        else:
-            return False
-
-
-    def test_init(self):
-        # default values
-        action1 = task.PythonAction(self._func_par)
-        assert action1.args == []
-        assert action1.kwargs == {}
-
-        # not a callable
-        py.test.raises(task.InvalidTask, task.PythonAction, "abc")
-        # args not a list
-        py.test.raises(task.InvalidTask, task.PythonAction, self._func_par, "c")
-        # kwargs not a list
-        py.test.raises(task.InvalidTask, task.PythonAction,
-                      self._func_par, None, "a")
-
-
-    def test_functionParametersArgs(self):
-        action = task.PythonAction(self._func_par,args=(2,2,25))
-        action.execute()
-
-    def test_functionParametersKwargs(self):
-        action = task.PythonAction(self._func_par,
-                              kwargs={'par1':2,'par2':2,'par3':25})
-        action.execute()
-
-    def test_functionParameters(self):
-        action = task.PythonAction(self._func_par,args=(2,2),
-                                   kwargs={'par3':25})
-        action.execute()
-
-    def test_functionParametersFail(self):
-        action = task.PythonAction(self._func_par, args=(2,3),
-                                   kwargs={'par3':25})
-        py.test.raises(TaskFailed, action.execute)
-
-    def test_str(self):
-        def str_sample(): return True
-        action = task.PythonAction(str_sample)
-        assert "Python: function str_sample" == str(action), "'%s'"%str(action)
-
-    def test_repr(self):
-        def repr_sample(): return True
-        action = task.PythonAction(repr_sample)
-        assert  "<PythonAction: '%s'>" % repr(repr_sample) == repr(action)
-
-    def test_result(self):
-        def vvv(): return "my value"
-        action = task.PythonAction(vvv)
-        action.execute()
-        assert "my value" == action.result
-
-    def test_values(self):
-        def vvv(): return {'x': 5, 'y':10}
-        action = task.PythonAction(vvv)
-        action.execute()
-        assert {'x': 5, 'y':10} == action.values
-
-
-class TestPythonVerbosity(object):
-    def write_stderr(self):
-        sys.stderr.write("this is stderr S\n")
-
-    def write_stdout(self):
-        sys.stdout.write("this is stdout S\n")
-
-    def test_captureStderr(self):
-        action = task.PythonAction(self.write_stderr)
-        action.execute()
-        assert "this is stderr S\n" == action.err, repr(action.err)
-
-    def test_captureStdout(self):
-        action = task.PythonAction(self.write_stdout)
-        action.execute()
-        assert "this is stdout S\n" == action.out, repr(action.out)
-
-    def test_noCaptureStderr(self, capsys):
-        action = task.PythonAction(self.write_stderr)
-        action.execute(err=sys.stderr)
-        got = capsys.readouterr()[1]
-        assert "this is stderr S\n" == got, repr(got)
-
-    def test_noCaptureStdout(self, capsys):
-        action = task.PythonAction(self.write_stdout)
-        action.execute(out=sys.stdout)
-        got = capsys.readouterr()[0]
-        assert "this is stdout S\n" == got, repr(got)
-
-    def test_redirectStderr(self):
-        tmpfile = os.tmpfile()
-        action = task.PythonAction(self.write_stderr)
-        action.execute(err=tmpfile)
-        tmpfile.seek(0)
-        got = tmpfile.read()
-        tmpfile.close()
-        assert "this is stderr S\n" == got, got
-
-    def test_redirectStdout(self):
-        tmpfile = os.tmpfile()
-        action = task.PythonAction(self.write_stdout)
-        action.execute(out=tmpfile)
-        tmpfile.seek(0)
-        got = tmpfile.read()
-        tmpfile.close()
-        assert "this is stdout S\n" == got, got
-
-
-##############
-
-
-class TestCreateAction(object):
-    def testBaseAction(self):
-        class Sample(task.BaseAction): pass
-        action = task.create_action(Sample())
-        assert isinstance(action, Sample)
-
-    def testStringAction(self):
-        action = task.create_action("xpto 14 7")
-        assert isinstance(action, task.CmdAction)
-
-    def testMethodAction(self):
-        def dumb(): return
-        action = task.create_action(dumb)
-        assert isinstance(action, task.PythonAction)
-
-    def testTupleAction(self):
-        def dumb(): return
-        action = task.create_action((dumb,[1,2],{'a':5}))
-        assert isinstance(action, task.PythonAction)
-
-    def testInvalidActionNone(self):
-        py.test.raises(task.InvalidTask, task.create_action, None)
-
-    def testInvalidActionObject(self):
-        py.test.raises(task.InvalidTask, task.create_action, self)
-
-
 
 class TestTaskCheckInput(object):
 
@@ -332,11 +57,12 @@ class TestTask(object):
 
     # dependency types going to the write place
     def test_dependencyTypes(self):
-        dep = ["file1.txt",":taskX","file2", "?res1"]
+        dep = ["file1.txt",":taskX","file2", "?res1", ":stu*"]
         t = task.Task("MyName", ["MyAction"], dep)
         assert t.task_dep == [dep[1][1:], dep[3][1:]]
         assert t.file_dep == [dep[0],dep[2]]
         assert t.result_dep == [dep[3][1:]]
+        assert t.wild_dep == [dep[4][1:]]
 
     def test_dependencyTrueRunonce(self):
         t = task.Task("Task X",["taskcmd"], dependencies=[True])
@@ -355,7 +81,7 @@ class TestTask(object):
         py.test.raises(task.InvalidTask, task.Task,
                       "Task X",["taskcmd"], dependencies=[123])
 
-    def test_ruOnce_or_fileDependency(self):
+    def test_runOnce_or_fileDependency(self):
         py.test.raises(task.InvalidTask, task.Task,
                       "Task X",["taskcmd"], dependencies=[True,"whatever"])
 
@@ -377,6 +103,18 @@ class TestTask(object):
         assert 'p1-default' == t.options['p1']
         assert '' == t.options['p2']
 
+    def test_setup(self):
+        # 2 kinds of setup: old-object, new-task_name
+        class MySetup:pass
+        setup_obj = MySetup()
+        t = task.Task("task5", ['action'], setup=[setup_obj, "task2"])
+        assert [setup_obj] == t.setup
+        assert ["task2"] == t.setup_tasks
+
+    def test_run_status(self):
+        t = task.Task("t", ["q"])
+        assert t.run_status is None
+
 
 class TestTask_Getargs(object):
     def test_ok(self):
@@ -414,7 +152,8 @@ class TestTaskActions(object):
 
     def test_failure(self):
         t = task.Task("taskX", ["%s 1 2 3" % PROGRAM])
-        py.test.raises(TaskError, t.execute)
+        got = t.execute()
+        assert isinstance(got, TaskError)
 
     # make sure all cmds are being executed.
     def test_many(self):
@@ -427,11 +166,13 @@ class TestTaskActions(object):
 
     def test_fail_first(self):
         t = task.Task("taskX", ["%s 1 2 3" % PROGRAM, PROGRAM])
-        py.test.raises(TaskError, t.execute)
+        got = t.execute()
+        assert isinstance(got, TaskError)
 
     def test_fail_second(self):
         t = task.Task("taskX", ["%s 1 2" % PROGRAM, "%s 1 2 3" % PROGRAM])
-        py.test.raises(TaskError, t.execute)
+        got = t.execute()
+        assert isinstance(got, TaskError)
 
 
     # python and commands mixed on same task
@@ -446,6 +187,21 @@ class TestTaskActions(object):
         assert "hi_stdout_PY_hi_list" == got, repr(got)
 
 
+class TestTaskTeardown(object):
+    def test_ok(self):
+        got = []
+        def put(x):
+            got.append(x)
+        t = task.Task('t1', [], teardown=[(put, [1]), (put, [2])])
+        assert None == t.execute_teardown()
+        assert [1,2] == got
+
+    def test_fail(self):
+        def my_raise():
+            raise Exception('hoho')
+        t = task.Task('t1', [], teardown=[(my_raise,)])
+        got = t.execute_teardown()
+        assert isinstance(got, CatchedException)
 
 
 class TestTaskClean(object):
@@ -592,10 +348,10 @@ class TestDictToTask(object):
 
     def testDictFieldTypo(self):
         dict_ = {'name':'z','actions':['xpto 14'],'typo_here':['xxx']}
-        py.test.raises(task.InvalidTask, task.dict_to_task, dict_)
+        py.test.raises(action.InvalidTask, task.dict_to_task, dict_)
 
     def testDictMissingFieldAction(self):
-        py.test.raises(task.InvalidTask, task.dict_to_task, {'name':'xpto 14'})
+        py.test.raises(action.InvalidTask, task.dict_to_task, {'name':'xpto 14'})
 
 
 class TestCmdExpandAction(object):
@@ -639,9 +395,9 @@ class TestPythonActionPrepareKwargsMeta(object):
     def test_no_extra_args(self, task_depchanged):
         def py_callable():
             return True
-        action = task.PythonAction(py_callable)
-        action.task = task_depchanged
-        action.execute()
+        my_action = action.PythonAction(py_callable)
+        my_action.task = task_depchanged
+        my_action.execute()
 
     def test_keyword_extra_args(self, task_depchanged):
         got = []
@@ -649,9 +405,9 @@ class TestPythonActionPrepareKwargsMeta(object):
             got.append(kwargs['targets'])
             got.append(kwargs['dependencies'])
             got.append(kwargs['changed'])
-        action = task.PythonAction(py_callable)
-        action.task = task_depchanged
-        action.execute()
+        my_action = action.PythonAction(py_callable)
+        my_action.task = task_depchanged
+        my_action.execute()
         assert got == [['targets'], ['dependencies'], ['changed']], got
 
     def test_named_extra_args(self, task_depchanged):
@@ -660,9 +416,9 @@ class TestPythonActionPrepareKwargsMeta(object):
             got.append(targets)
             got.append(dependencies)
             got.append(changed)
-        action = task.PythonAction(py_callable)
-        action.task = task_depchanged
-        action.execute()
+        my_action = action.PythonAction(py_callable)
+        my_action.task = task_depchanged
+        my_action.execute()
         assert got == [['targets'], ['dependencies'], ['changed']]
 
     def test_mixed_args(self, task_depchanged):
@@ -671,9 +427,9 @@ class TestPythonActionPrepareKwargsMeta(object):
             got.append(a)
             got.append(b)
             got.append(changed)
-        action = task.PythonAction(py_callable, ('a', 'b'))
-        action.task = task_depchanged
-        action.execute()
+        my_action = action.PythonAction(py_callable, ('a', 'b'))
+        my_action.task = task_depchanged
+        my_action.execute()
         assert got == ['a', 'b', ['changed']]
 
     def test_extra_arg_overwritten(self, task_depchanged):
@@ -682,9 +438,9 @@ class TestPythonActionPrepareKwargsMeta(object):
             got.append(a)
             got.append(b)
             got.append(changed)
-        action = task.PythonAction(py_callable, ('a', 'b', 'c'))
-        action.task = task_depchanged
-        action.execute()
+        my_action = action.PythonAction(py_callable, ('a', 'b', 'c'))
+        my_action.task = task_depchanged
+        my_action.execute()
         assert got == ['a', 'b', 'c']
 
     def test_extra_kwarg_overwritten(self, task_depchanged):
@@ -693,16 +449,16 @@ class TestPythonActionPrepareKwargsMeta(object):
             got.append(a)
             got.append(b)
             got.append(kwargs['changed'])
-        action = task.PythonAction(py_callable, ('a', 'b'), {'changed': 'c'})
-        action.task = task_depchanged
-        action.execute()
+        my_action = action.PythonAction(py_callable, ('a', 'b'), {'changed': 'c'})
+        my_action.task = task_depchanged
+        my_action.execute()
         assert got == ['a', 'b', 'c']
 
     def test_extra_arg_default_disallowed(self, task_depchanged):
         def py_callable(a, b, changed=None): pass
-        action = task.PythonAction(py_callable, ('a', 'b'))
-        action.task = task_depchanged
-        py.test.raises(task.InvalidTask, action.execute)
+        my_action = action.PythonAction(py_callable, ('a', 'b'))
+        my_action.task = task_depchanged
+        py.test.raises(task.InvalidTask, my_action.execute)
 
 class TestPythonActionOptions(object):
     def test_task_options(self):
